#! /bin/sh
#
#   Module name 		: install.sh
#   Date of last delta		: 10/25/10
#   Release number		: 3.13
#
#   Copyright (c) Transoft Ltd 2003
########################################################
#
# install.sh: Uncompresses the installation then runs
#	     through some initial installation events.
#
########################################################
#
# RedHat Linux BASH is not compiled USG, this causes that the \escape
# sequences in echo to fail using the builtin echo.
# The /bin/echo is only OK for V6.0, not V6.1 so use function to enable the
# \escape sequences (echo -e) of the bash echo.
# Also remove aliases that may cause problems.
#
os=`uname -s`
if [ "x${os}" = "xLinux" ]
then
    echotest=`echo "Test\tTest"`
    if [ "${echotest}" = "Test\\tTest" ]
    then
	echo()
	{
	    /bin/echo -e $*
	    return $?
	}
    fi
    unalias ls 2>/dev/null
    unalias rm 2>/dev/null
    unalias mv 2>/dev/null
    unalias cp 2>/dev/null
    unalias ps 2>/dev/null
fi

# Get installation components into path
PATH=${PATH}:`pwd`/instdir
export PATH

TMP=${TEMPDIR-/tmp}


# Checks that the user is the superuser
SuCheck()
{
    #If ${proc} = 0 then Superuser.       
    AtTransoft=0
    proc=`id | awk -F= '{ print $2 }' | awk -F\( '{ print $1 }'`

    if [ ${proc} -ne 0 ]
    then
	if [ -d /u/macros/prodinfo/sccs ]
	then
	    echo "\n\nWARNING: you are not root! *****************\n\n"
	    # allow script to run without being root
	    # for safe testing.
	    AtTransoft=1
	else
	    echo "\nYou must be the superuser to run this program."
	    exit 1
	fi
    fi     
}   

# warn if usqlsd_* files in bin folder
WarnUsqlsd()
{
   ls bin/usqlsd_* >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
        echo "\nYou are advised to rename manual copy of usqlsd_*"
        echo "to something else otherwise this will be mistaken for"
        echo "current valid usqlsd"
        echo "Do you wish to continue ? (y/[n]): \c"
        read answer
           if [ "${answer}" != "y" -a "${answer}" != "Y" ]
           then
           echo "\nYou can re-run install.sh after renaming usqlsd_*"
           exit 1
           fi
        fi
}

# Checks to see that TCP is installed and enabled.
# Also check added to advise if usqlsd is running
CheckTCP()
{
    for PS_OPTION in -e ax -ax			# System V, BSD, hybrid.
    do
	ps ${PS_OPTION} 2>/dev/null | grep usqlsd | grep -v 'grep' > /dev/null
	if [ $? -eq 0 ]
	then
	    echo "You are advised to close down existing users of this product"
	    echo "before installing an upgrade to the product."
	    echo "Do you wish to continue ? (y/[n]): \c"
	    read answer
	    if [ "${answer}" != "y" -a "${answer}" != "Y" ]
	    then 
		echo "\nYou can re-run install.sh when product not in use" 
    		exit 1
	    fi
	fi

	ps ${PS_OPTION} 2>/dev/null | grep inetd | grep -v 'grep' > /dev/null
	INETD=$?

	if [ ${INETD} -eq 0 ]
	then
	    break
	fi
    done

    if [ ${INETD} -eq 1 ]
    then
	echo "TCP/IP is either not installed or enabled on your machine,"
	echo "do you wish to continue ? (y/[n]): \c"
	read answer

	if [ "${answer}" != "y" -a "${answer}" != "Y" ]
	then 
	    echo "\nYou can re-run install.sh when TCP/IP is installed" 
	    exit 1
	fi

	echo "\nYou need to install TCP/IP before this product will work\n" 
    fi
}

# save the usqlsd.ini file
PreserveIni()
{
    if [ -f ${path}/bin/usqlsd.ini -a -f ${path}/bin/usqlsd ]
    then
	cp ${path}/bin/usqlsd.ini $TMP/usqlsd.ini.$$
	cp ${path}/bin/usqlsd $TMP/usqlsd.$$
	return 1
    else
	return 0
    fi
}

RestoreIni()
{
    if [ -f $TMP/usqlsd.ini.$$ ]
    then
	mv -f $TMP/usqlsd.ini.$$ ${path}/bin/usqlsd.ini
	rm -f $TMP/usqlsd.ini.$$
    else
	echo Unable to restore usqlsd.ini file
    fi
}

RestoreSD()
{
    if [ -f $TMP/usqlsd.$$ ]
    then
	#
	# If ${path}/bin/usqlsd is a symbolic link, it is converted
	# into a file so that the original server is preserved on
	# future attempts to install the product.
	#
	mv -f $TMP/usqlsd.$$ ${path}/bin/usqlsd
	rm -f $TMP/usqlsd.$$
	chmod 755 ${path}/bin/usqlsd
	echo "\nYour original copy of usqlsd has been reinstated"
    else
	echo Unable to restore usqlsd file
    fi
}

CheckIni()
{
    cd ${path}/bin > /dev/null
    ./usqlsd -s
    if [ $? -ne 0 ]
    then
       echo "\nThe install has detected a INVALID entry in the usqlsd.ini file."
       echo "The details are in ${path}/bin/usqlcs.log."
       echo "You need to correct this before re-installing this product."
       RestoreSD
       exit 1
    fi
    cd ${path}
}

# Note that the usqlcs[0-9].z files are written on the discs in reverse order
# So the first .z file extracted will have the highest number and the last
# disc will contain usqlcs1.z.

special=0

CheckExtractComplete()
{
    fname=`ls *.z 2> /dev/null | sed -n "1p"`
    if [ "${fname}" = "usqlcs.z" ]
    then
	# no splitting of discs has occured
	return 0
    fi
    lastfile=`ls *.z 2>/dev/null | tail -1`
    if [ -z "${lastfile}" ]
    then
	# This can happen if disc 1 just has install.sh on it, 2nd
	# disc has compressed file on it.
	echo "\nDisk 2 has not been loaded."
	special=1
	return 2
    fi
    ndiscs=`echo ${lastfile} | sed "s/usqlcs//;s/\.z$//"`
    count=${ndiscs}
    fail=0
    while [ ${count} -ne 0 ]
    do
	if [ ! -f usqlcs${count}.z ]
	then
	    miss=`expr ${ndiscs} + 1 - ${count} + ${special}`
	    echo "\nDisk ${miss} has not been loaded."
	    if [ "${fail}" -eq 0 ]
	    then
		fail=${miss}
	    fi
	fi
	count=`expr ${count} - 1`
    done
    if [ ${fail} -ne 0 ]
    then
	echo "\nExtract of media not complete."
    fi
    return ${fail}
}

GetDiscList()
{
    x=`tar 2>&1 | grep dev/rfd`
    if [ -n "${x}" ]
    then
	# must be on SCO or another platform that lists devices
	tar 2>&1 | awk '{ print $2 }' | grep fd
    fi
    ls /dev/fd[0-9] 2>/dev/null			# most unix's
    ls /dev/rdsk/c2d[0-9]s2 2>/dev/null		# HP unix
    ls /dev/rfd[0-9]c 2>/dev/null		# OSF 1
    ls /vol/dev/rdiskette0/* 2>/dev/null	# Sun OS
    ls /dev/rdsk/f03ht 2>/dev/null		# SV - Unixware etc.
}

# Note that the list is numbered from 0 to make the numbering compatible
# with the SCO unix tar command that lists from 0.
ChooseDevice()
{
    Tfile=$TMP/disc.list.$$
    GetDiscList > ${Tfile}
    DevNam="none"

    while [ ! -b ${DevNam} -a ! -c ${DevNam} ]
    do
	if [ "${DevNam}" != "none" ]
	then
	    echo "\n'${DevNam}' is invalid, try again.\n"
	fi
	echo "\nWhat device do you want to extract from?\n"
	echo "Enter a number or a device name.\n"
	Count=-1
	for i in `cat ${Tfile}`
	do
	    Count=`expr ${Count} + 1`
	    echo "\t\t"${Count}")\t"${i}
	done

	echo "\nDevice? \c"
	read DevNam
	if [ -z "${DevNam}" ]
	then
	    DevNam="none"
	else
	    case ${DevNam} in
	    [0-9]* )
		if [ ${DevNam} -ge 0 -a ${DevNam} -le ${Count} ]
		then
		    position=`expr ${DevNam} + 1`
		    DevNam=`cat ${Tfile} | sed -n "${position}p"`
		fi
		;;
	    /* )
		;;
	    * )	# Invalid option
		DevNam="invalid"
		;;
	    esac
	fi
    done
    rm -f ${Tfile}
    echo ${DevNam} > .disc_dev
}

LoadADisc()
{
    if [ -f /bin/volcheck ]
    then
	# for Sun
	Sun=/bin/volcheck
	${Sun}
	Eject="eject floppy"
    else
	Sun=''
    fi

    if [ ! -f .disc_dev ]
    then
	ChooseDevice
    fi

    echo "\nAbout to read from disk device "`cat .disc_dev`
    echo "\nInsert disk "$1" into drive and press"
    echo "\t'c' to change device name"
    echo "\t'q' to quit"
    echo "or\tRETURN to load disk"
    echo "\n> \c"
    read ans

    if [ "x${ans}" = "xc" ]
    then
	ChooseDevice
    fi

    if [ "x${ans}" = "xq" -o "x${ans}" = "xQ" ]
    then
	echo "\nInstall aborted."
	exit 1
    fi

    if [ -n "${Sun}"  ]
    then
	${Sun}
    fi

    echo "\nExecuting  tar -xvf "`cat .disc_dev`
    tar -xvf `cat .disc_dev`
    Ret=$?
    if [ -n "${Eject}" ]
    then
	${Eject}
    fi
    return ${Ret}
}

DoUncomp()
{
	if [ "x${os}" = "xLinux" ]
	then
		gunzip
	else
		# Not sure what the "-c -" is about.
		uncompress -c -
	fi
}

ExtractFiles()
{
    finished=0
    Tanswer=''
    answer=''
    while [ ${finished} -eq 0 ]
    do
	CheckExtractComplete
	MissDisc=$?
	if [ ${MissDisc} -eq 0 ]
	then
	    finished=1
	else
	    LoadADisc ${MissDisc}
	fi
    done

    echo "\nUncompressing the tar file..."
    if [ -f usqlcs1.z ]
    then
	if [ -f usqlcs10.z ]
	then
	    cat usqlcs[0-9].z usqlcs[1-9][0-9].z > usqlcs.z
	else
	    cat usqlcs[0-9].z > usqlcs.z
	fi
	if [ $? -ne 0 ]
	then
	    echo "FAILED\n\nUnable to re-combine Archive file\n"
	    if [ ${INIFILE} -ne 0 ]
	    then
		echo "Restoring orginal usqlsd & usqlsd.ini file.\n"
		RestoreIni
		RestoreSD
	    fi
	    exit 1
	fi
	rm -f usqlcs[0-9].z usqlcs[1-9][0-9].z
    fi
    if [ -d instdir ]
    then
	chmod 755 instdir
	ls instdir/* >/dev/null 2>&1
	if [ $? -eq 0 ]
	then
	    chmod 755 instdir/*
	fi
    fi
    DoUncomp < usqlcs.z > /dev/null 2> $TMP/install.er.$$
    if [ $? -ne 0 -o -s $TMP/install.er.$$ ]
    then
	rm -f $TMP/install.er.$$
	if [ -f kript.u -a ! -f kript ]
	then
	    echo "\nExtracting program to decode using a product key ..."
	    uudecode kript.u
	    ret=$?
	fi
	if [ -f ./kript ]
	then
	    rm -f $TMP/install.er.$$
	    DispGetPasswd
	    ./kript ${answer} < usqlcs.z > $TMP/usqlcs.z.$$ 2> $TMP/install.er.$$
	    ret=$?
	    if [ ${ret} -eq 0 ]; then
		DoUncomp < $TMP/usqlcs.z.$$ > $TMP/usqlcs.$$ 2> $TMP/install.er.$$
		ret=$?
		if [ ${ret} -eq 0 ]; then
# HPUX Tar sends -v output to stderr, not stdout so errors won't go to file
#		    tar -xvf $TMP/usqlcs.$$ 2> $TMP/install.er.$$
	    	    tar -xvf $TMP/usqlcs.$$
		    ret=$?
		fi
	    fi
	    rm -f $TMP/usqlcs.$$ $TMP/usqlcs.z.$$
	else
	    echo "Decription Program not available" > $TMP/install.er.$$
	    ret=1
	fi
    else
	rm -f $TMP/install.er.$$
	DoUncomp < usqlcs.z > $TMP/usqlcs.$$ 2> $TMP/install.er.$$
	ret=$?
	if [ ${ret} -eq 0 ]; then
# HPUX Tar sends -v output to stderr, not stdout so errors won't go to file
#	    tar -xvf $TMP/usqlcs.$$ 2> $TMP/install.er.$$
	    tar -xvf $TMP/usqlcs.$$
	    ret=$?
	fi
	rm -f $TMP/usqlcs.$$
	ret=$?
    fi

    if [ ${ret} -ne 0 -o -s $TMP/install.er.$$ -o ! -d instdir ]
    then
    	cat $TMP/install.er.$$
	if [ "${Tanswer}" = "${answer}" ]
	then
	    echo "\nUncompress failed... check for password error\n"
	else
	    echo "\nUncompress failed...\n"
	fi
	if [ ${INIFILE} -ne 0 ]
	then
	    echo "Restoring orginal usqlsd & usqlsd.ini file.\n"
	    RestoreIni
	    RestoreSD
	fi
	rm -f $TMP/install.er.$$
	exit 1
    fi

    # Now delete the TAR file(s) and any warnings from TAR extract
    rm -f usqlcs.z $TMP/install.er.$$

    if [ -f readme ]
    then
	mv -f readme README
    fi

    chmod 555 instdir instdir/*

    . instcomp.sh

    if [ "${AtTransoft}" -ne 1 ]
    then
	CorrectPerms
    fi

    VerifyChecksums

    InitialSetup

    #
    # test -L does not work on Solaris 2.6 !!!!
    # so we use 'if [ "`ls -l <xx> 2>/dev/null | cut -c1`" != "l" ]'
    # instead of 'if [ ! -L <xx> ]'
    #
    if [ "`ls -l bin/usqlsd 2>/dev/null | cut -c1`" != "l" ]
    then
	ls bin/usqlsd_* >/dev/null 2>&1
	if [ $? -eq 0 -a -f bin/usqlsd ]
	then
	    mv bin/usqlsd bin/usqlsd.bu
	fi
	MultiBinCheck
    fi
}

DispGetPasswd()
{
    answer=""
    while [ -z "${answer}" ]
    do
	echo "\nYou need the product password obtained from either the shipment"
	echo "form or your supplier(this is not the product licence)."
	echo "Enter 'q' to quit - you can then install the product later"
	echo "\nPlease enter the product password: \c"
	read answer
	if [ "${answer}" = "q" -o "${answer}" = "'q'" -o "${answer}" = "Q" ]
	then
	    echo "\nInstall aborted."
	    if [ ${INIFILE} -ne 0 ]
	    then
		echo "Restoring orginal usqlsd & usqlsd.ini file.\n"
		RestoreIni
	        RestoreSD
	    fi
	    exit 1
	fi
	Tanswer=`echo "${answer}" | cut -f3 -d\- `
	if [ -n "${Tanswer}" ]
	then
	    answer=${Tanswer}
	fi
    done
}

# ------------------------ start of the script -------------------------

clear
echo "==================================================================="
echo "T R A N S O F T   U / S Q L   S E R V E R   I N S T A L L A T I O N"
echo "==================================================================="

# find the path of the installed software
path=`pwd`

# check we are superuser
SuCheck

# warn if usqlsd_* files in bin folder
WarnUsqlsd

# check that tcp is installed
CheckTCP

# preserve the usqlsd & usqlsd.ini file.
PreserveIni
INIFILE=$?

# Only uncompress the files if this is the first time in
# i.e. do the .z files exist !
ls usqlcs[0-9].z > /dev/null 2>&1
if [ ! -d instdir -o -f usqlcs.z -o $? -eq 0 ]
then
    ExtractFiles
    if [ ${INIFILE} -ne 0 ]
    then
	echo "Restoring orginal usqlsd.ini file.\n"
	RestoreIni
	CheckIni
    fi
else
    if [ ! -f ./instdir/instcomp.sh ]
    then
	echo "One of the instalation scripts is missing, Can't continue."
	exit 1
    fi
    chmod 555 ./instdir/instcomp.sh
    . instcomp.sh
fi

CheckFilePermissions
echo "File permissions checked - OK" >> ${path}/install.data

# Display or print the README file.
#DispReadme

######################################################
# Do these things only once at install time ! 
#########################################################

cd ${path}/bin

# run through the initial set up procedure
clear
echo "Running through the set up procedures..."
echo

FindPort
echo "Usable socket port found - OK" >> ${path}/install.data

WriteStartScript
echo "Server start script written - OK" >> ${path}/install.data

# check for auto start of usqlsd at boot-up option
rc2Check

if [ -f ${path}/csm_type.txt ]
then
    echo "\nRefer to csm_type.txt for details of current UFD data types."
fi

if [ "$AtTransoft" -ne 1 ]
then
    chmod 444 ${path}/install.data
fi

echo "\nThe Server installation is complete.\n"

# Check for and run OEM Install
if [ -x ${path}/oem.sh ]; then
    exec ${path}/oem.sh ${path}
fi

exit 0
